// This file is part of a hashlife-based simulator for the CBlocks3D environment written by Will Stevens
// You may freely use and distribute this program.

#ifndef CB3DE_H
#define CB3DE_H

#include <fstream>

typedef enum {empty,wire,nor,slide,fuse,unfuse,rotate} blockType;

typedef enum {north,east,south,west,front,back,none} direction;

typedef enum {northmask = 1, eastmask = 2, southmask = 4, westmask = 8, frontmask = 16, backmask = 32} directionMasks;

typedef struct
{
  int x,y,z;
} vector;

#define BLOCK_BITS 18
// The bits used in block are as follows:
// 0-2 block type
// 3-7 orientation
// 8 output
// 9-14 connectivity
// 15-17 move propagation direction (direction+1)
//
// 0 represents an empty cell
typedef unsigned block;


unsigned getBlockType(block b);
void setBlockType(block *b, unsigned tp);

unsigned getOrientation(block b);
void setOrientation(block *b, unsigned orientation);

unsigned getOutput(block b);
void setOutput(block *b, unsigned output);

unsigned getConnect(block b);
void setConnect(block *b, unsigned connect);

unsigned getMovedir(block b);
void setMovedir(block *b, unsigned d);

void alterConnect(block *b, unsigned d, bool c);

#define EMPTY_FLAG		1
#define NOTPROP_FLAG	2
#define PROPSTATIC_FLAG 4
#define VISITED_FLAG	128

#define CALCINDEX_CALC	 0
#define CALCINDEX_CALC2	 1

#define CALCTYPE_NORMAL	0
#define CALCTYPE_PROP	1
#define CALCTYPE_DOMOVE	2

typedef struct otNode
{
	unsigned char level;
	unsigned char flags;
	otNode *calc[2];

	union
	{
		block leaf;
		struct otNode *children[2][2][2];
	} u;

	struct otNode *next; // for use in hash and also in freeList
} otNode;

#ifndef CB3DE_CPP
extern vector dirVectorLookup[6];

extern direction opposite[6];

// For looking up what frontDir should be, given northDir and eastDir
extern direction frontDirLookup[6][6];

// given a 5 bit orientation value, this table stores the northDir, eastDir and frontDir
extern direction dirLookup[24][3];

extern otNode *eg;
#endif

unsigned lookupOrientationValue(direction p, direction s);

otNode *createNode(unsigned level, bool populated, bool lookupEmpty);
otNode *duplicateNode(otNode *o);
void destroyNode(otNode *o);

void initTables(void);
otNode *loadBlock(block b, int x, int y, int z, otNode *o);
otNode *load(std::ifstream &is);
void save(otNode *eg, std::ofstream &os, int xo, int yo, int zo);

otNode *singleStep(otNode *eg, unsigned &iters);

void Draw(otNode *eg, int xo, int yo, int zo);


#endif