// This file is part of a hashlife-based simulator for the CBlocks3D environment written by Will Stevens
// You may freely use and distribute this program.

#include <stdio.h>
#include <time.h>

#define NODRAW

#include "cb3de.h"
#include "universe.h"

CUniverse U;

// parameters are: <.grd input> <iterations> <iterations per dump> <dump prefix>
int main(int argc, char *argv[])
{
	if (argc != 5)
	{
	  fprintf(stderr,"<.grd input> <iterations> <iterations per dump> <dump prefix>");
	  return -1;
	}
	
	std::ifstream is(argv[1]);
	
	U.Load(is);
	
	unsigned iters = atoi(argv[2]);
	unsigned itersPerDump = atoi(argv[3]);
	unsigned lastDump = 0;

	printf("Started:%d\n",time(NULL));
	
	for(; U.iters<iters;)
	{		            
		if (U.iters - lastDump >= itersPerDump)
		{
			char fname[32];
			
			sprintf(fname,"%s%d",argv[4],U.iters);
			
			std::ofstream os(fname);
			U.Save(os);

			lastDump = (U.iters/itersPerDump)*itersPerDump;
		}
		
		U.Process();
	}

	{
		char fname[32];
			
		sprintf(fname,"%s%d",argv[4],U.iters);
			
		std::ofstream os(fname);
		U.Save(os);
	}

    printf("Finished:%d\n",time(NULL));
	
	return 0;
}
